//+-----------------------------------------------------------------------------
//  MS Bookmaker
//  Copyright (c) 1998-2000  Microsoft Corporation.  All Rights Reserved.
//
//  File:       ErrHandler.cpp
//
//  Contents:   sample implementation of ILITCallback
//
//------------------------------------------------------------------------------
#include "LITConvertDemo.h"
#include "litgen_msgcodes.h"

// nothing interesting about the IUnknown implementations...
STDMETHODIMP ErrHandler::QueryInterface(REFIID riid, void** ppv)
{
    HRESULT hr = S_OK;
    if (!ppv)
    {
        hr = E_INVALIDARG;
        goto Out;
    }
    *ppv = 0;
    if (riid == IID_ILITCallback || riid == IID_IUnknown)
        *ppv = static_cast<ILITCallback*>(this);
    if (!*ppv)
    {
        hr = E_NOINTERFACE;
        goto Out;
    }
    reinterpret_cast<IUnknown*>(*ppv)->AddRef();
Out:
    return hr;
}

STDMETHODIMP_(ULONG) ErrHandler::AddRef()
{
    return InterlockedIncrement(&m_lRef);
}

STDMETHODIMP_(ULONG) ErrHandler::Release()
{
    long ret = InterlockedDecrement(&m_lRef);
    if (!ret)
        delete this;
    return ret;
}

//+-----------------------------------------------------------------------------
//
//  Function:       Message
//  Description:    receives diagnostic messages from LITGen
//  Parameters:     iType - message type (error or warning, usually)
//                  iMessageCode - exact message number
//                  wszText - message suitable (usually) for display to a user
//
//------------------------------------------------------------------------------
STDMETHODIMP ErrHandler::Message(int iType, int iMessageCode, const wchar_t* pwszText)
{
    switch(iType)
    {
    case -1:
        // LITGEN_MSG_MISC_PAGEBREAKS is a message that always follows the success
        // message.  In some situations, this is unimportant; we'll use it to show
        // how to selectively suppress messages.
        if (iMessageCode != LITGEN_MSG_MISC_PAGEBREAKS)
            printf("LITGen message (%d): %S\n", iMessageCode, pwszText);
        break;
    case 0:
        fprintf(stderr, "LITGen error (%d): %S\n", iMessageCode, pwszText);
        break;
    case 1:
        fprintf(stderr, "LITGen warning (%d): %S\n", iMessageCode, pwszText);
        break;
    }
    return S_OK;
}

