//+-----------------------------------------------------------------------------
//  MS Bookmaker
//  Copyright (c) 1998-2000  Microsoft Corporation.  All Rights Reserved.
//
//  File:       FileCopier.cpp
//
//  Contents:   two unrelated but very similar stream copying routines
//
//------------------------------------------------------------------------------
#include "LITConvertDemo.h"

void CopyCSS(const _bstr_t& bstrFilename, ILITCSSHost* pHost)
{
    HRESULT hr = S_OK;

    HANDLE hIn = INVALID_HANDLE_VALUE;
    try
    {
        // LITGen will provide absolute paths to all files, in a degenerate URI
        // form; the path will contain a protocol prefix, but any escaped
        // characters will have been expanded.  For example, if the package
        // stream contained "foo%20bar.css", the path we'd get here might be
        // "file://C:\foo\foo bar.css".
        // For this sample, we'll only support local files.  Strip off the
        // file://, if present (otherwise, just let CreateFile() fail).
        const char* pszFilename = bstrFilename;
        if (!strncmp(pszFilename, "file://", 7))
            pszFilename += 7;
        hIn = CreateFile(pszFilename, GENERIC_READ, FILE_SHARE_READ, 0,
            OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, 0);
        if (INVALID_HANDLE_VALUE == hIn)
        {
            hr = HRESULT_FROM_WIN32(GetLastError());
            CheckHR(hr, "Error 0x%x opening CSS file %S.\n", bstrFilename);
        }
        for(;;)
        {
            DWORD cb;
            char achBuffer[512];
            if (ReadFile(hIn, achBuffer, sizeof(achBuffer), &cb, 0))
            {
                if (cb)
                {
                    hr = pHost->Write((BYTE*)achBuffer, cb, 0);
                    CheckHR(hr, "Error 0x%x copying CSS from %S.\n",
                        bstrFilename);
                }
                else
                    break;
            }
            else
            {
                hr = HRESULT_FROM_WIN32(GetLastError());
                CheckHR(hr, "Error 0x%x reading from CSS file %S.\n",
                    bstrFilename);
            }
        }
        CloseHandle(hIn);
    }
    catch(...)
    {
        CloseHandle(hIn);
        throw;
    }
}

void CopyImage(const _bstr_t& bstrFilename, ILITImageHost* pHost)
{
    HRESULT hr = S_OK;

    HANDLE hIn = INVALID_HANDLE_VALUE;
    try
    {
        // For this sample, we'll only support local files.  Strip off the
        // file://, if present (otherwise, just let CreateFile() fail).
        const char* pszFilename = bstrFilename;
        if (!strncmp(pszFilename, "file://", 7))
            pszFilename += 7;

        hIn = CreateFile(pszFilename, GENERIC_READ, FILE_SHARE_READ, 0,
            OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, 0);
        if (INVALID_HANDLE_VALUE == hIn)
        {
            hr = HRESULT_FROM_WIN32(GetLastError());
            CheckHR(hr, "Error 0x%x opening image file %S.\n", bstrFilename);
        }
        for(;;)
        {
            DWORD cb;
            char achBuffer[512];
            if (ReadFile(hIn, achBuffer, sizeof(achBuffer), &cb, 0))
            {
                if (cb)
                {
                    hr = pHost->Write((BYTE*)achBuffer, cb, 0);
                    CheckHR(hr, "Error 0x%x copying image from %S.\n",
                        bstrFilename);
                }
                else
                    break;
            }
            else
            {
                hr = HRESULT_FROM_WIN32(GetLastError());
                CheckHR(hr, "Error 0x%x reading from image file %S.\n",
                    bstrFilename);
            }
        }
        CloseHandle(hIn);
    }
    catch(...)
    {
        CloseHandle(hIn);
        throw;
    }
}
